import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import greenfoot.junitUtils.jupiter.runner.GreenfootRunner;

import greenfoot.junitUtils.EventDispatch;

/**
 * Test Driven Development (TDD)
 * 
 * @author Francisco Guerra
 */
@ExtendWith(GreenfootRunner.class)
class CrabTest {

    CrabWorld crabWorld;
    Crab crab;
    
    @BeforeEach
    void setUp() throws Exception {
        // GIVEN a crab world with its crab
        crabWorld = new CrabWorld();
        crab = new Crab();
        crabWorld.addObject(crab, 231, 203);
    }

    @Test
    void testGetX() throws Exception {
        // WHEN nothing
        
        // THEN the crab is still in the same place
        assertEquals(231, crab.getX());
    }

    @Test
    void testGetY() throws Exception {
        // WHEN nothing
        
        // THEN the crab is still in the same place
        assertEquals(203, crab.getY());
    }
    
    @Test
    void testGetRotation() throws Exception {
        // WHEN nothing
        
        // THEN the crab is still in the same direction
        assertEquals(0, crab.getRotation());
    }
    
    @Test
    void testGetImage() throws Exception {
        // WHEN nothing
        
        // THEN the crab has its initial image
        assertEquals("crab.png", crab.getImage().getImageFileName());
    }
    
    @Test
    void testTwoMoveAtEdge() throws Exception {
        // GIVEN the edge position
        int x = crab.getWorld().getWidth()-1;
        int y = 100;
        crab.setLocation(x, y);
        
        
        // WHEN any key is not typed
        crab.act();
        crab.act();
        
        // THEN the crab moves 5
        assertEquals(  x, crab.getX());
        assertEquals(y, crab.getY());
        assertEquals( 0, crab.getRotation());
    }
    
    @Test
    void testTurnRight() throws Exception {
        // GIVEN the initial position
        assertEquals(0, crab.getRotation());
        
        // WHEN right key is typed
        EventDispatch.keyTyped("right");
        crab.act();
        
        // THEN the crab rotates 4 degrees to the right
        assertEquals(4, crab.getRotation());
    }

    @Test
    void testTurnRightBis() throws Exception {
        // GIVEN the initial position
        int degrees = crab.getRotation();
        
        // WHEN right key is typed
        EventDispatch.keyTyped("right");
        crab.act();
        
        // THEN the crab rotates 4 degrees to the right
        assertEquals((360+degrees+4)%360, crab.getRotation());
    }

    @Test
    void testTurnRightBisBis() throws Exception {
        // GIVEN the initial position
        crab.setRotation(10);
        
        // WHEN right key is typed
        EventDispatch.keyTyped("right");
        crab.act();
        
        // THEN the crab rotates 4 degrees to the right
        assertEquals(14, crab.getRotation());
    }

    @Test
    void testTurnLeft() throws Exception {
        // GIVEN the initial position
        assertEquals(0, crab.getRotation());
        
        // WHEN left key is typed
        EventDispatch.keyTyped("left");
        crab.act();
        
        // THEN the crab rotates 4 degrees to the left
        assertEquals(356, crab.getRotation());
    }

    @Test
    void testTurnLeftBis() throws Exception {
        // GIVEN the initial position
        int degrees = crab.getRotation();
        
        // WHEN left key is typed
        EventDispatch.keyTyped("left");
        crab.act();
        
        // THEN the crab rotates 4 degrees to the left
        assertEquals((360+degrees-4)%360, crab.getRotation());
    }

    @Test
    void testTurnLeftBisBis() throws Exception {
        // GIVEN the initial position
        crab.setRotation(10);
        
        // WHEN left key is typed
        EventDispatch.keyTyped("left");
        crab.act();
        
        // THEN the crab rotates 4 degrees to the left
        assertEquals(6, crab.getRotation());
    }

    @Test
    void testTurnRight90() throws Exception {
        // GIVEN a crab that has rotated 90 degrees to the right
        for (int k=0; k<90/4;k++){
            EventDispatch.keyTyped("right");
            crab.act();
        }
        
        // WHEN any key is not typed
        int x = crab.getX();
        int y = crab.getY();
        crab.act();
        
        // THEN the crab goes down 5
        assertEquals(  x, crab.getX());
        assertEquals(y+5, crab.getY());
    }

    @Test
    void testTurnRight180() throws Exception {
        // GIVEN a crab that has rotated 180 degrees to the right
        for (int k=0; k<180/4;k++){
            EventDispatch.keyTyped("right");
            crab.act();
        }
        
        // WHEN any key is not typed
        int x = crab.getX();
        int y = crab.getY();
        crab.act();
        
        // THEN the crab goes back 5
        assertEquals(x-5, crab.getX());
        assertEquals(  y, crab.getY());
    }

    @Test
    void testTurnRight270() throws Exception {
        // GIVEN a crab that has rotated 270 degrees to the right
        for (int k=0; k<270/4;k++){
            EventDispatch.keyTyped("right");
            crab.act();
        }
        
        // WHEN any key is not typed
        int x = crab.getX();
        int y = crab.getY();
        crab.act();
        
        // THEN the crab goes up 5
        assertEquals(  x, crab.getX());
        assertEquals(y-5, crab.getY());
    }

    @Test
    void testTurnLeft90() throws Exception {
        // GIVEN a crab that has rotated 90 degrees to the left
        for (int k=0; k<90/4;k++){
            EventDispatch.keyTyped("left");
            crab.act();
        }
        
        // WHEN any key is not typed
        int x = crab.getX();
        int y = crab.getY();
        crab.act();
        
        // THEN the crab goes up 5
        assertEquals(  x, crab.getX());
        assertEquals(y-5, crab.getY());
    }

    @Test
    void testTurnLeft180() throws Exception {
        // GIVEN a crab that has rotated 180 degrees to the left
        for (int k=0; k<180/4;k++){
            EventDispatch.keyTyped("left");
            crab.act();
        }
        
        // WHEN any key is not typed
        int x = crab.getX();
        int y = crab.getY();
        crab.act();
        
        // THEN the crab goes back 5
        assertEquals(x-5, crab.getX());
        assertEquals(  y, crab.getY());
    }

    @Test
    void testTurnLeft270() throws Exception {
        // GIVEN a crab that has rotated 270 degrees to the left
        for (int k=0; k<270/4;k++){
            EventDispatch.keyTyped("left");
            crab.act();
        }
        
        // WHEN any key is not typed
        int x = crab.getX();
        int y = crab.getY();
        crab.act();
        
        // THEN the crab goes down 5
        assertEquals(  x, crab.getX());
        assertEquals(y+5, crab.getY());
    }
    
    @Test
    void testLookForWorm() throws Exception {
        // GIVEN a worm in the same position as the crab
        Worm worm = new Worm();
        int x = crab.getX();
        int y = crab.getY();
        crabWorld.addObject(worm, x, y);
        assertTrue(crabWorld.getObjectsAt(x, y, Worm.class).contains(worm));

        // WHEN the crab checks that there is a worm
        crab.act();
        
        // THEN you can listen to how the crab eats the worm
        assertFalse(crabWorld.getObjectsAt(x, y, Worm.class).contains(worm));
    }
}
